/* This example illustrates simple reads and writes to a USB RAW device as results user 
 * events .  It is designed to only work with USB RAW devices.
 * 
 * To run this example, install a USB RAW device and configure Measurement & Automation 
 * Explorer (MAX) to recognize the USB device (see Tutorial: Using NI-VISA 3.0 to 
 * Control Your USB Device at www.ni.com).  After the USB RAW device is successfully 
 * installed and configured, and you have opened a session to the device, you can 
 * perform control in and control out functions with the USB RAW device. 
 * 
 * This example allows the user to cause USB control pipe transfers to or from a USB 
 * device. The values of the data payload in the setup stage of the control transfer 
 * are taken as parameters and include bmRequestType, bRequest, wValue, wIndex, and 
 * wLength.  The value of each in these calls is largely dependant on the USB RAW 
 * device, however there are a few standard calls that can be made.  
 * 
 * For example, to get the status of a particular device, you could use the following 
 * values in a Control In function:
 * Request Type: 0x80 (Device to Host, Standard Type, Device Recipient)
 * Request: 0x0 (GET_STATUS)
 * Value: 0x0 
 * Index: 0x0
 * Length: 0x2
 * NOTE: All inputs above are configured to accept a HEXADECIMAL input.  This can be 
 * changed to accept decimal values by setting the HEXADECIMAL property to FALSE for each
 * input.
 * 
 * The last two bits of the value returned will let you know whether the device is set 
 * to remotely wake up (bit 1), and whether the device is self powered (bit 0).
 * 
 * To perform control transfers with another device, close the session to the current 
 * device, select another device from the VISA resource name control, and open a session 
 * to it.
 * 
 * When complete, close the application to end execution.
*/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using NationalInstruments.VisaNS;


namespace NationalInstruments.Examples.SimpleReadWrite
{
    /// <summary>
    /// Summary description for Form1.
    /// </summary>
    public class MainForm : System.Windows.Forms.Form
    {
        private UsbRaw mySession;
        private string lastResourceString = null;
        private System.Windows.Forms.TextBox readText;
        private System.Windows.Forms.Button openSessionButton;
        private System.Windows.Forms.Button clearButton;
        private System.Windows.Forms.Button closeSessionButton;
        private System.Windows.Forms.Label stringToReadLabel;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox writeText;
		private System.Windows.Forms.Button controlInButton;
		private System.Windows.Forms.Button controlOutButton;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.NumericUpDown requestTypeNumeric;
		private System.Windows.Forms.NumericUpDown lengthNumeric;
		private System.Windows.Forms.NumericUpDown indexNumeric;
		private System.Windows.Forms.NumericUpDown dataNumeric;
		private System.Windows.Forms.NumericUpDown requestNumeric;
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public MainForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();
            SetupControlState(false);
									
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if(mySession != null)
                {
                    mySession.Dispose();
                }
                if (components != null) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.controlInButton = new System.Windows.Forms.Button();
			this.openSessionButton = new System.Windows.Forms.Button();
			this.readText = new System.Windows.Forms.TextBox();
			this.clearButton = new System.Windows.Forms.Button();
			this.closeSessionButton = new System.Windows.Forms.Button();
			this.stringToReadLabel = new System.Windows.Forms.Label();
			this.controlOutButton = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.writeText = new System.Windows.Forms.TextBox();
			this.requestTypeNumeric = new System.Windows.Forms.NumericUpDown();
			this.lengthNumeric = new System.Windows.Forms.NumericUpDown();
			this.indexNumeric = new System.Windows.Forms.NumericUpDown();
			this.dataNumeric = new System.Windows.Forms.NumericUpDown();
			this.requestNumeric = new System.Windows.Forms.NumericUpDown();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.requestTypeNumeric)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.lengthNumeric)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.indexNumeric)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.dataNumeric)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.requestNumeric)).BeginInit();
			this.SuspendLayout();
			// 
			// controlInButton
			// 
			this.controlInButton.Location = new System.Drawing.Point(126, 38);
			this.controlInButton.Name = "controlInButton";
			this.controlInButton.Size = new System.Drawing.Size(111, 23);
			this.controlInButton.TabIndex = 3;
			this.controlInButton.Text = "Control In";
			this.controlInButton.Click += new System.EventHandler(this.ControlInButton_Click);
			// 
			// openSessionButton
			// 
			this.openSessionButton.Location = new System.Drawing.Point(8, 5);
			this.openSessionButton.Name = "openSessionButton";
			this.openSessionButton.Size = new System.Drawing.Size(111, 23);
			this.openSessionButton.TabIndex = 0;
			this.openSessionButton.Text = "Open Session";
			this.openSessionButton.Click += new System.EventHandler(this.openSession_Click);
			// 
			// readText
			// 
			this.readText.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.readText.Location = new System.Drawing.Point(9, 260);
			this.readText.Multiline = true;
			this.readText.Name = "readText";
			this.readText.ReadOnly = true;
			this.readText.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.readText.Size = new System.Drawing.Size(224, 45);
			this.readText.TabIndex = 6;
			this.readText.TabStop = false;
			this.readText.Text = "";
			// 
			// clearButton
			// 
			this.clearButton.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.clearButton.Location = new System.Drawing.Point(5, 315);
			this.clearButton.Name = "clearButton";
			this.clearButton.Size = new System.Drawing.Size(231, 24);
			this.clearButton.TabIndex = 7;
			this.clearButton.Text = "Clear";
			this.clearButton.Click += new System.EventHandler(this.clear_Click);
			// 
			// closeSessionButton
			// 
			this.closeSessionButton.Location = new System.Drawing.Point(126, 5);
			this.closeSessionButton.Name = "closeSessionButton";
			this.closeSessionButton.Size = new System.Drawing.Size(111, 23);
			this.closeSessionButton.TabIndex = 1;
			this.closeSessionButton.Text = "Close Session";
			this.closeSessionButton.Click += new System.EventHandler(this.closeSession_Click);
			// 
			// stringToReadLabel
			// 
			this.stringToReadLabel.Location = new System.Drawing.Point(9, 245);
			this.stringToReadLabel.Name = "stringToReadLabel";
			this.stringToReadLabel.Size = new System.Drawing.Size(167, 15);
			this.stringToReadLabel.TabIndex = 9;
			this.stringToReadLabel.Text = "Control In Buffer:";
			// 
			// controlOutButton
			// 
			this.controlOutButton.Location = new System.Drawing.Point(9, 38);
			this.controlOutButton.Name = "controlOutButton";
			this.controlOutButton.Size = new System.Drawing.Size(111, 23);
			this.controlOutButton.TabIndex = 12;
			this.controlOutButton.Text = "Control Out";
			this.controlOutButton.Click += new System.EventHandler(this.ControlOutButton_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(9, 190);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(167, 15);
			this.label1.TabIndex = 13;
			this.label1.Text = "Control Out Buffer:";
			// 
			// writeText
			// 
			this.writeText.Location = new System.Drawing.Point(9, 205);
			this.writeText.Name = "writeText";
			this.writeText.Size = new System.Drawing.Size(221, 20);
			this.writeText.TabIndex = 14;
			this.writeText.Text = "*idn?\\n";
			// 
			// requestTypeNumeric
			// 
			this.requestTypeNumeric.Hexadecimal = true;
			this.requestTypeNumeric.Location = new System.Drawing.Point(9, 84);
			this.requestTypeNumeric.Maximum = new System.Decimal(new int[] {
																			   255,
																			   0,
																			   0,
																			   0});
			this.requestTypeNumeric.Name = "requestTypeNumeric";
			this.requestTypeNumeric.Size = new System.Drawing.Size(111, 20);
			this.requestTypeNumeric.TabIndex = 15;
			// 
			// lengthNumeric
			// 
			this.lengthNumeric.Hexadecimal = true;
			this.lengthNumeric.Location = new System.Drawing.Point(9, 164);
			this.lengthNumeric.Maximum = new System.Decimal(new int[] {
																		  65535,
																		  0,
																		  0,
																		  0});
			this.lengthNumeric.Name = "lengthNumeric";
			this.lengthNumeric.Size = new System.Drawing.Size(111, 20);
			this.lengthNumeric.TabIndex = 17;
			// 
			// indexNumeric
			// 
			this.indexNumeric.Hexadecimal = true;
			this.indexNumeric.Location = new System.Drawing.Point(127, 124);
			this.indexNumeric.Maximum = new System.Decimal(new int[] {
																		 65535,
																		 0,
																		 0,
																		 0});
			this.indexNumeric.Name = "indexNumeric";
			this.indexNumeric.Size = new System.Drawing.Size(111, 20);
			this.indexNumeric.TabIndex = 18;
			// 
			// dataNumeric
			// 
			this.dataNumeric.Hexadecimal = true;
			this.dataNumeric.Location = new System.Drawing.Point(9, 124);
			this.dataNumeric.Maximum = new System.Decimal(new int[] {
																		65535,
																		0,
																		0,
																		0});
			this.dataNumeric.Name = "dataNumeric";
			this.dataNumeric.Size = new System.Drawing.Size(111, 20);
			this.dataNumeric.TabIndex = 19;
			// 
			// requestNumeric
			// 
			this.requestNumeric.Hexadecimal = true;
			this.requestNumeric.Location = new System.Drawing.Point(126, 84);
			this.requestNumeric.Maximum = new System.Decimal(new int[] {
																		   255,
																		   0,
																		   0,
																		   0});
			this.requestNumeric.Name = "requestNumeric";
			this.requestNumeric.Size = new System.Drawing.Size(111, 20);
			this.requestNumeric.TabIndex = 20;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(9, 68);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(100, 15);
			this.label2.TabIndex = 21;
			this.label2.Text = "Request Type";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(9, 149);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(100, 15);
			this.label3.TabIndex = 22;
			this.label3.Text = "Length";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(126, 109);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(100, 15);
			this.label4.TabIndex = 23;
			this.label4.Text = "Index";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(9, 109);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(100, 15);
			this.label5.TabIndex = 24;
			this.label5.Text = "Data";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(126, 68);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(100, 15);
			this.label6.TabIndex = 25;
			this.label6.Text = "Request";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(243, 349);
			this.Controls.Add(this.label6);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.requestNumeric);
			this.Controls.Add(this.dataNumeric);
			this.Controls.Add(this.indexNumeric);
			this.Controls.Add(this.lengthNumeric);
			this.Controls.Add(this.requestTypeNumeric);
			this.Controls.Add(this.writeText);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.controlOutButton);
			this.Controls.Add(this.stringToReadLabel);
			this.Controls.Add(this.closeSessionButton);
			this.Controls.Add(this.clearButton);
			this.Controls.Add(this.readText);
			this.Controls.Add(this.openSessionButton);
			this.Controls.Add(this.controlInButton);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimumSize = new System.Drawing.Size(251, 383);
			this.Name = "MainForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "RAW USB Control In/Out";
			((System.ComponentModel.ISupportInitialize)(this.requestTypeNumeric)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.lengthNumeric)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.indexNumeric)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.dataNumeric)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.requestNumeric)).EndInit();
			this.ResumeLayout(false);

		}
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {
            Application.Run(new MainForm());
        }

        private void openSession_Click(object sender, System.EventArgs e)
        {
            using (SelectResource sr = new SelectResource())
            {
                if(lastResourceString != null)
                {
                    sr.ResourceName = lastResourceString;
                }
                DialogResult result = sr.ShowDialog(this);
                if(result == DialogResult.OK)
                {
                    lastResourceString = sr.ResourceName;
                    Cursor.Current = Cursors.WaitCursor;
                    try
                    {
                        mySession = (UsbRaw)ResourceManager.GetLocalManager().Open(sr.ResourceName);
                        SetupControlState(true);
                    }
                    catch(InvalidCastException)
                    {
                        MessageBox.Show("Resource selected must be a USB session");
                    }
                    catch(Exception exp)
                    {
                        MessageBox.Show(exp.Message);
                    }
                    finally
                    {
                        Cursor.Current = Cursors.Default;
                    }
                }
            }
        }

        private void closeSession_Click(object sender, System.EventArgs e)
        {
            SetupControlState(false);
            mySession.Dispose();
        }

        private void clear_Click(object sender, System.EventArgs e)
        {
            readText.Text = String.Empty;
        }

        private void SetupControlState(bool isSessionOpen)
        {
            openSessionButton.Enabled = !isSessionOpen;
            closeSessionButton.Enabled = isSessionOpen;
			controlInButton.Enabled = isSessionOpen;
			controlOutButton.Enabled = isSessionOpen;
		    clearButton.Enabled = isSessionOpen;

            if(isSessionOpen)
            {
                readText.Text = String.Empty;
            }
        }
       
		private void ControlOutButton_Click(object sender, System.EventArgs e)
		{
            try
			{
				char [] tmp = writeText.Text.ToCharArray();
				byte [] buffer = new byte[tmp.Length];
				
				for(int element=0; element<tmp.Length; element++)
					buffer[element] = Convert.ToByte(tmp[element]);

				mySession.ControlOut(
					Convert.ToInt16(requestTypeNumeric.Value),
					Convert.ToInt16(requestNumeric.Value),
					Convert.ToInt16(dataNumeric.Value),
					Convert.ToInt16(indexNumeric.Value),
					buffer);				
			}

			catch(Exception exp)
			{
				MessageBox.Show(exp.Message);
			}
		
		}

		private void ControlInButton_Click(object sender, System.EventArgs e)
		{
			
			try
			{
				byte [] buffer = mySession.ControlIn(
					Convert.ToInt16(requestTypeNumeric.Value), 
					Convert.ToInt16(requestNumeric.Value),
					Convert.ToInt16(dataNumeric.Value), 
					Convert.ToInt16(indexNumeric.Value),
					Convert.ToInt16(lengthNumeric.Value));

				readText.Text = String.Empty;
				for(int i=0; i<buffer.Length; i++)
					readText.Text = readText.Text + " " + buffer[i].ToString();										
			}

			catch(Exception exp)
			{
				MessageBox.Show(exp.Message);
			}
		}

    }
}
